import asyncio
import aiohttp
import json
import websockets

from .sense_api import *
from .sense_exceptions import *

class ASyncSenseable(SenseableBase):
    
    async def authenticate(self, username, password):
        auth_data = {
            "email": username,
            "password": password
        }

        # Get auth token
        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(API_URL+'authenticate',
                                        data=auth_data) as resp:

                    # check for 200 return
                    if resp.status != 200:
                        raise SenseAuthenticationException(
                            "Please check username and password. API Return Code: %s" %
                            resp.status)

                    # Build out some common variables
                    self.set_auth_data(await resp.json())
        except Exception as e:
            raise Exception('Connection failure: %s' % e)
                
    # Update the realtime data for asyncio
    async def update_realtime(self):
        # rate limit API calls
        if self._realtime and self.rate_limit and \
           self.last_realtime_call + self.rate_limit > time():
            return self._realtime
        self.last_realtime_call = time()
        await self.async_realtime_stream(single=True)
    
    async def async_realtime_stream(self, callback=None, single=False):
        """ Reads realtime data from websocket"""
        url = WS_URL % (self.sense_monitor_id, self.sense_access_token)
        # hello, features, [updates,] data
        async with websockets.connect(url) as ws:
            while True:
                try:
                    message = await asyncio.wait_for(
                        ws.recv(), timeout=self.wss_timeout)
                except asyncio.TimeoutError:
                    raise SenseAPITimeoutException("API websocket timed out")
                
                result = json.loads(message)
                if result.get('type') == 'realtime_update':
                    data = result['payload']
                    self.set_realtime(data)
                    if callback: callback(data)
                    if single: return
            
    async def get_realtime_future(self, callback):
        """ Returns an async Future to parse realtime data with callback"""
        await self.async_realtime_stream(callback)
        
    async def api_call(self, url, payload={}):
        timeout = aiohttp.ClientTimeout(total=self.api_timeout)
        async with aiohttp.ClientSession() as session:
            async with session.get(API_URL + url,
                          headers=self.headers,
                          timeout=timeout,
                          data=payload) as resp:
                return await resp.json()
        # timed out
        raise SenseAPITimeoutException("API call timed out") 

    async def get_trend_data(self, scale):
        if scale.upper() not in valid_scales:
            raise Exception("%s not a valid scale" % scale)
        t = datetime.now().replace(hour=12)
        json = self.api_call(
            'app/history/trends?monitor_id=%s&scale=%s&start=%s' %
            (self.sense_monitor_id, scale, t.isoformat()))
        self._trend_data[scale] = await json

    async def update_trend_data(self):
        for scale in valid_scales:
            await self.get_trend_data(scale)

    async def get_discovered_device_names(self):
        # lots more info in here to be parsed out
        json = self.api_call('app/monitors/%s/devices' %
                                 self.sense_monitor_id)
        self._devices = await [entry['name'] for entry in json]
        return self._devices

    async def get_discovered_device_data(self):
        json = self.api_call('monitors/%s/devices' %
                             self.sense_monitor_id)
        return await json




import json
import sys
from time import time
from datetime import datetime

from .sense_exceptions import *

API_URL = 'https://api.sense.com/apiservice/api/v1/'
WS_URL = "wss://clientrt.sense.com/monitors/%s/realtimefeed?access_token=%s"
API_TIMEOUT = 5
WSS_TIMEOUT = 5
RATE_LIMIT = 30

# for the last hour, day, week, month, or year
valid_scales = ['HOUR', 'DAY', 'WEEK', 'MONTH', 'YEAR']


class SenseableBase(object):

    def __init__(self, username=None, password=None,
                 api_timeout=API_TIMEOUT, wss_timeout=WSS_TIMEOUT):

        # Timeout instance variables
        self.api_timeout = api_timeout
        self.wss_timeout = wss_timeout
        
        self._realtime = {}
        self._devices = []
        self._trend_data = {}        
        for scale in valid_scales: self._trend_data[scale] = {}

        if username and password:
            self.authenticate(username, password)

    def set_auth_data(self, data):
        self.sense_access_token = data['access_token']
        self.sense_user_id = data['user_id']
        self.sense_monitor_id = data['monitors'][0]['id']

        # create the auth header
        self.headers = {'Authorization': 'bearer {}'.format(
            self.sense_access_token)}
        

    @property
    def devices(self):
        """Return devices."""
        return self._devices
    
    def set_realtime(self, data):
        self._realtime = data
        self.last_realtime_call = time()
        
    def get_realtime(self):
        return self._realtime     

    @property
    def active_power(self):
        return self._realtime.get('w', 0)

    @property
    def active_solar_power(self):
        return self._realtime.get('solar_w', 0)

    @property
    def active_voltage(self):
        return self._realtime.get('voltage', 0)
    
    @property
    def active_frequency(self):
        return self._realtime.get('hz', 0)
    
    @property
    def daily_usage(self):
        return self.get_trend('DAY', False)

    @property
    def daily_production(self):
        return self.get_trend('DAY', True)
    
    @property
    def weekly_usage(self):
        # Add today's usage
        return self.get_trend('WEEK', False)

    @property
    def weekly_production(self):
        # Add today's production
        return self.get_trend('WEEK', True)
    
    @property
    def monthly_usage(self):
        # Add today's usage
        return self.get_trend('MONTH', False)

    @property
    def monthly_production(self):
        # Add today's production
        return self.get_trend('MONTH', True)
    
    @property
    def yearly_usage(self):
        # Add this month's usage
        return self.get_trend('YEAR', False)

    @property
    def yeary_production(self):
        # Add this month's production
        return self.get_trend('YEAR', True)

    @property
    def active_devices(self):
        return [d['name'] for d in self._realtime.get('devices', {})]

    def get_trend(self, scale, is_production):
        key = "production" if is_production else "consumption"       
        if key not in self._trend_data[scale]: return 0
        total = self._trend_data[scale][key].get('total', 0)
        if scale == 'WEEK' or scale == 'MONTH':
            return total + self.get_trend('DAY', is_production)
        if scale == 'YEAR':
            return total + self.get_trend('MONTH', is_production)
        return total



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name = 'sense_energy',
    packages = ['sense_energy'], 
    install_requires=[
        'requests',
        'websocket-client',
        'websockets;python_version>="3.5"',
    ],
    version = '0.7.0',
    description = 'API for the Sense Energy Monitor',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author = 'scottbonline',
    author_email = 'scottbonline@gmail.com',
    url = 'https://github.com/scottbonline/sense',
    download_url = 'https://github.com/scottbonline/sense/archive/0.7.0.tar.gz', 
    keywords = ['sense', 'energy', 'api', 'pytest'], 
    classifiers = [
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
)
